@echo off
rem *********************************************************************
rem *                                                                   *
rem * NOME SCRIPT: AnalisiAD.cmd                                        *
rem *                                                                   *
rem * Script scritto da Alessandro Tani (Home Works S.p.A.)             *
rem *                                                                   *
rem * Versione 1.0 - Modificato l'ultima volta il 18/08/2009            *
rem *                da Alessandro Tani                                 *
rem * SCOPO:                                                            *
rem *                                                                   *
rem *  Questo script svolge un'analisi diagniostica di Active Directory *
rem *                                                                   *
rem * File di log:                                                      *
rem *                                                                   *
rem *  Nello script vengono riportati i vari file di log utilizzati     *
rem *                                                                   *
rem * PREREQUISITI:                                                     *
rem *                                                                   *
rem *  Lo script deve venire eseguito da una persona che fa parte dei   *
rem *  gruppi Domain Admins ed Enterprise Admins. Lo script va eseguito *
rem *  solamente su postazioni Windows 2003 (nessuna prova  stata      *
rem *  ancora fatta su Windows 2008) che ricoprono il ruolo di Domain   *
rem *  Controller. I Support Tools di Windows devono essere installati  *
rem *  sulla postazione in cui viene eseguito lo script.                *
rem *                                                                   *
rem * TESTO:                                                            *
rem *                                                                   *
rem *  Nessuno                                                          *
rem *                                                                   *
rem *********************************************************************

rem Definiamo l'ambiente locale
setlocal enableextensions
echo.

rem Controlliamo che esistano le variabili d'ambiente necessarie
if "%OSLanguage%" equ "" (
	echo La variabile d'ambiente "OSLanguage" non e' stata impostata
	echo per far girare questo script e' necessario impostare questa
	echo variabile. Questa variabile deve essere impostata ad IT se
	echo il sistema operativo e' in Italiano ed ad EN se il sistema
	echo operativo e' in Inglese.
	goto END
)

rem Impostiamo le variabili
echo Questo script svolge un'analisi diagniostica di Active Directory.
echo Nome del dominio DNS analizzato: %USERDNSDOMAIN%
echo.
set /p NomeCliente="Inserisci il nome della societa' (senza inserire spazi): "

rem Controlliamo l'esistenza della cartella Logs
if not exist %SystemDrive%\Logs md %SystemDrive%\Logs

rem Calcoliamo la data e l'ora di esecuzione dello script
if /i "%OSLanguage%" equ "EN" for /f "Tokens=1-4 Delims=/ " %%i in ('date /t') do set DATA=%%l%%k%%j_%%i
if /i "%OSLanguage%" equ "IT" for /f "Tokens=1-4 Delims=/ " %%i in ('date /t') do set DATA=%%k%%j%%i

rem Eseguiamo i controlli in Active Directory
echo.
echo Eseguiamo i controlli di Active Directory ...
echo.

if not exist "%ProgramFiles%\Support Tools\DCDiag.exe" (
	echo.
	echo  # Il file "%ProgramFiles%\Support Tools\DCDiag.exe" non esiste!
	echo  # Pertanto non potranno essere eseguiti i controlli: 
	echo  #   - DCDiag.exe  /e /v /c
	echo  #   - DCDiag.exe /test:DNS /e /v
	echo  # Per risolvere il problema provvedere ad installare i "Windows Support Tools"
	echo  # http://www.microsoft.com/downloads/details.aspx?displaylang=en^&FamilyID=96a35011-fd83-419d-939b-9a772ea2df90
	echo.
	goto STEP2
)
set /a I=1
:DCDIAGFULL
if not exist %SystemDrive%\Logs\%DATA%_%NomeCliente%_DCDiag_Full_Test_#%I%.log (
	echo Eseguiamo il comando DCDiag.exe  /e /v /c ...
	DCDiag.exe  /e /v /c > %SystemDrive%\Logs\%DATA%_%NomeCliente%_DCDiag_Full_Test_#%I%.log
) else (
	set /a I=I+1
	goto DCDIAGFULL
)

set /a I=1
:DCDIAGDNS
if not exist %SystemDrive%\Logs\%DATA%_%NomeCliente%_DCDiag_DNS_Test_#%I%.log (
	echo Eseguiamo il comando DCDiag.exe /test:DNS /e /v ...
	DCDiag /test:DNS /e /v > %SystemDrive%\Logs\%DATA%_%NomeCliente%_DCDiag_DNS_Test_#%I%.log
) else (
	set /a I=I+1
	goto DCDIAGDNS
)

:STEP2
if not exist "%ProgramFiles%\Support Tools\Netdiag.exe" (
	echo.
	echo  # Il file "%ProgramFiles%\Support Tools\Netdiag.exe" non esiste!
	echo  # Pertanto non potra' essere eseguito il controllo: Netdiag.exe
	echo  # Per risolvere il problema provvedere ad installare i "Windows Support Tools"
	echo  # http://www.microsoft.com/downloads/details.aspx?displaylang=en^&FamilyID=96a35011-fd83-419d-939b-9a772ea2df90
	echo.
	goto STEP3
)
set /a I=1
:NETDIAG
if not exist %SystemDrive%\Logs\%DATA%_%NomeCliente%_NetDiag_Test_#%I%.log (
	echo Eseguiamo il comando Netdiag.exe ...
	Netdiag > %SystemDrive%\Logs\%DATA%_%NomeCliente%_NetDiag_Test_#%I%.log
) else (
	set /a I=I+1
	goto NETDIAG
)

:STEP3
if not exist "%ProgramFiles%\Support Tools\NetDom.exe" (
	echo.
	echo  # Il file "%ProgramFiles%\Support Tools\NetDom.exe" non esiste!
	echo  # Pertanto non potra' essere eseguito il controllo: NetDom.exe Query FSMO
	echo  # Per risolvere il problema provvedere ad installare i "Windows Support Tools"
	echo  # http://www.microsoft.com/downloads/details.aspx?displaylang=en^&FamilyID=96a35011-fd83-419d-939b-9a772ea2df90
	echo.
	goto STEP4
)
set /a I=1
:NETDOM
if not exist %SystemDrive%\Logs\%DATA%_%NomeCliente%_FSMO_Test_#%I%.log (
	echo Eseguiamo il comando NetDom.exe Query FSMO ...
	NetDom Query FSMO > %SystemDrive%\Logs\%DATA%_%NomeCliente%_FSMO_Test_#%I%.log
) else (
	set /a I=I+1
	goto NETDOM
)

:STEP4
if not exist "%ProgramFiles%\Support Tools\nltest.exe" (
	echo.
	echo  # Il file "%ProgramFiles%\Support Tools\nltest.exe" non esiste!
	echo  # Pertanto non potra' essere eseguito il controllo: nltest.exe /dsgetdc:%USERDNSDOMAIN%
	echo  # Per risolvere il problema provvedere ad installare i "Windows Support Tools"
	echo  # http://www.microsoft.com/downloads/details.aspx?displaylang=en^&FamilyID=96a35011-fd83-419d-939b-9a772ea2df90
	echo.
	goto STEP5
)
set /a I=1
:NLTEST
if not exist %SystemDrive%\Logs\%DATA%_%NomeCliente%_Domain_Controller_State_Test_#%I%.log (
	echo Eseguiamo il comando nltest.exe /dsgetdc:%USERDNSDOMAIN% ...
	nltest /dsgetdc:%USERDNSDOMAIN% > %SystemDrive%\Logs\%DATA%_%NomeCliente%_Domain_Controller_State_Test_#%I%.log
) else (
	set /a I=I+1
	goto NLTEST
)

:STEP5
if not exist "%ProgramFiles%\Support Tools\repadmin.exe" (
	echo.
	echo  # Il file "%ProgramFiles%\Support Tools\repadmin.exe" non esiste!
	echo  # Pertanto non potranno essere eseguiti i controlli: 
	echo  #   - repadmin.exe /showrepl %COMPUTERNAME%.%USERDNSDOMAIN%
	echo  #   - repadmin /showconn %COMPUTERNAME%.%USERDNSDOMAIN%
	echo  #   - repadmin /replsummary * /Errorsonly /sort:Delta
	echo  #   - repadmin /Showrepl * /CSV
	echo  # Per risolvere il problema provvedere ad installare i "Windows Support Tools"
	echo  # http://www.microsoft.com/downloads/details.aspx?displaylang=en^&FamilyID=96a35011-fd83-419d-939b-9a772ea2df90
	echo.
	goto STEP6
)
set /a I=1
:REPADMIN#1
if not exist %SystemDrive%\Logs\%DATA%_%NomeCliente%_Replica_Partner_Test_#%I%.log (
	echo Eseguiamo il comando repadmin.exe /showrepl %COMPUTERNAME%.%USERDNSDOMAIN% ...
	repadmin /showrepl %COMPUTERNAME%.%USERDNSDOMAIN% > %SystemDrive%\Logs\%DATA%_%NomeCliente%_Replica_Partner_Test_#%I%.log
) else (
	set /a I=I+1
	goto REPADMIN#1
)

set /a I=1
:REPADMIN#2
if not exist %SystemDrive%\Logs\%DATA%_%NomeCliente%_Connection_Object_Test_#%I%.log (
	echo Eseguiamo il comando repadmin /showconn %COMPUTERNAME%.%USERDNSDOMAIN% ...
	repadmin /showconn %COMPUTERNAME%.%USERDNSDOMAIN% > %SystemDrive%\Logs\%DATA%_%NomeCliente%_Connection_Object_Test_#%I%.log
) else (
	set /a I=I+1
	goto REPADMIN#2
)

set /a I=1
:REPADMIN#3
if not exist %SystemDrive%\Logs\%DATA%_%NomeCliente%_Replication_Status_Test_#%I%.log (
	echo Eseguiamo il comando repadmin /replsummary * /Errorsonly /sort:Delta ...
	repadmin /replsummary * /Errorsonly /sort:Delta > %SystemDrive%\Logs\%DATA%_%NomeCliente%_Replication_Status_Test_#%I%.log
) else (
	set /a I=I+1
	goto REPADMIN#3
)

set /a I=1
:REPADMIN#4
if not exist %SystemDrive%\Logs\%DATA%_%NomeCliente%_Replication_Partners_Status_#%I%.csv (
	echo Eseguiamo il comando repadmin /Showrepl * /CSV ...
	repadmin /Showrepl * /CSV > %SystemDrive%\Logs\%DATA%_%NomeCliente%_Replication_Partners_Status_#%I%.csv
) else (
	set /a I=I+1
	goto REPADMIN#4
)

:STEP6
if not exist "%windir%\system32\dsquery.exe" (
	echo.
	echo  # Il file "%windir%\system32\dsquery.exe" non esiste!
	echo  # Pertanto non potranno essere eseguiti i controlli: 
	echo  #   - dsquery.exe server -forest -isgc
	echo  #   - dsquery.exe server -forest -hasfsmo pdc
	echo  #   - dsquery.exe server -forest -hasfsmo schema
	echo  #   - dsquery.exe server -forest -hasfsmo name
	echo  #   - dsquery.exe server -forest -hasfsmo infr
	echo  #   - dsquery.exe server -forest -hasfsmo rid
	echo  # Per risolvere il problema provvedere ad installare i "Windows Support Tools"
	echo  # http://www.microsoft.com/downloads/details.aspx?displaylang=en^&FamilyID=96a35011-fd83-419d-939b-9a772ea2df90
	echo.
	goto STEP7
)
set /a I=1
:DSQUERY#1
if not exist %SystemDrive%\Logs\%DATA%_%NomeCliente%_Global_Catalogs_List_#%I%.log (
	echo Eseguiamo il comando dsquery.exe server -forest -isgc ...
	echo +--------------------------------------------------------------+  > %SystemDrive%\Logs\%DATA%_%NomeCliente%_Global_Catalogs_List_#%I%.log
	echo ^|  ELENCO DEI DOMAIN CONTROLLER CHE SONO ANCHE GLOBAL CATALOG  ^| >> %SystemDrive%\Logs\%DATA%_%NomeCliente%_Global_Catalogs_List_#%I%.log
	echo +--------------------------------------------------------------+ >> %SystemDrive%\Logs\%DATA%_%NomeCliente%_Global_Catalogs_List_#%I%.log
	echo. >> %SystemDrive%\Logs\%DATA%_%NomeCliente%_Global_Catalogs_List_#%I%.log
	echo Nome Dominio Active Directory: %USERDNSDOMAIN% >> %SystemDrive%\Logs\%DATA%_%NomeCliente%_Global_Catalogs_List_#%I%.log
	echo Nome Computer: %COMPUTERNAME%.%USERDNSDOMAIN% >> %SystemDrive%\Logs\%DATA%_%NomeCliente%_Global_Catalogs_List_#%I%.log
	echo. >> %SystemDrive%\Logs\%DATA%_%NomeCliente%_Global_Catalogs_List_#%I%.log
	dsquery server -forest -isgc >> %SystemDrive%\Logs\%DATA%_%NomeCliente%_Global_Catalogs_List_#%I%.log
) else (
	set /a I=I+1
	goto DSQUERY#1
)

set /a I=1
:DSQUERY#2
if not exist %SystemDrive%\Logs\%DATA%_%NomeCliente%_FSMO_List_#%I%.log (
	echo +----------------------------------------------------------------------+  > %SystemDrive%\Logs\%DATA%_%NomeCliente%_FSMO_List_#%I%.log
	echo ^|  UBICAZIONE ^(SITE^) DEI DOMAIN CONTROLLER CHE RICOPRONO I RUOLI FSMO  ^| >> %SystemDrive%\Logs\%DATA%_%NomeCliente%_FSMO_List_#%I%.log
	echo +----------------------------------------------------------------------+ >> %SystemDrive%\Logs\%DATA%_%NomeCliente%_FSMO_List_#%I%.log
	echo. >> %SystemDrive%\Logs\%DATA%_%NomeCliente%_FSMO_List_#%I%.log
	echo Nome Dominio Active Directory: %USERDNSDOMAIN% >> %SystemDrive%\Logs\%DATA%_%NomeCliente%_FSMO_List_#%I%.log
	echo Nome Computer: %COMPUTERNAME%.%USERDNSDOMAIN% >> %SystemDrive%\Logs\%DATA%_%NomeCliente%_FSMO_List_#%I%.log
	echo. >> %SystemDrive%\Logs\%DATA%_%NomeCliente%_FSMO_List_#%I%.log
	echo Eseguiamo il comando dsquery.exe server -forest -hasfsmo pdc ...
	echo PDC Emulator Domain Controller: >> %SystemDrive%\Logs\%DATA%_%NomeCliente%_FSMO_List_#%I%.log
	dsquery server -forest -hasfsmo pdc >> %SystemDrive%\Logs\%DATA%_%NomeCliente%_FSMO_List_#%I%.log
	echo. >> %SystemDrive%\Logs\%DATA%_%NomeCliente%_FSMO_List_#%I%.log
	echo Eseguiamo il comando dsquery.exe server -forest -hasfsmo schema ...
	echo Schema Operations Master Domain Controller: >> %SystemDrive%\Logs\%DATA%_%NomeCliente%_FSMO_List_#%I%.log
	dsquery server -forest -hasfsmo schema >> %SystemDrive%\Logs\%DATA%_%NomeCliente%_FSMO_List_#%I%.log
	echo. >> %SystemDrive%\Logs\%DATA%_%NomeCliente%_FSMO_List_#%I%.log
	echo Eseguiamo il comando dsquery.exe server -forest -hasfsmo name ...
	echo Domain Naming Master Domain Controller: >> %SystemDrive%\Logs\%DATA%_%NomeCliente%_FSMO_List_#%I%.log
	dsquery server -forest -hasfsmo name >> %SystemDrive%\Logs\%DATA%_%NomeCliente%_FSMO_List_#%I%.log
	echo. >> %SystemDrive%\Logs\%DATA%_%NomeCliente%_FSMO_List_#%I%.log
	echo Eseguiamo il comando dsquery.exe server -forest -hasfsmo infr ...
	echo Infrastructure Operations Master Domain Controller: >> %SystemDrive%\Logs\%DATA%_%NomeCliente%_FSMO_List_#%I%.log
	dsquery server -forest -hasfsmo infr >> %SystemDrive%\Logs\%DATA%_%NomeCliente%_FSMO_List_#%I%.log
	echo. >> %SystemDrive%\Logs\%DATA%_%NomeCliente%_FSMO_List_#%I%.log
	echo Eseguiamo il comando dsquery.exe server -forest -hasfsmo rid ...
	echo RID Master Domain Controller: >> %SystemDrive%\Logs\%DATA%_%NomeCliente%_FSMO_List_#%I%.log
	dsquery server -forest -hasfsmo rid >> %SystemDrive%\Logs\%DATA%_%NomeCliente%_FSMO_List_#%I%.log
	echo. >> %SystemDrive%\Logs\%DATA%_%NomeCliente%_FSMO_List_#%I%.log
) else (
	set /a I=I+1
	goto DSQUERY#2
)

:STEP7
if not exist "%windir%\system32\W32tm.exe" (
	echo.
	echo  # Il file "%windir%\system32\W32tm.exe" non esiste!
	echo  # Pertanto non potra' essere eseguito il controllo: W32tm.exe /monitor
	echo  # Per risolvere il problema provvedere ad installare i "Windows Support Tools"
	echo  # http://www.microsoft.com/downloads/details.aspx?displaylang=en^&FamilyID=96a35011-fd83-419d-939b-9a772ea2df90
	echo.
	goto STEP8
)
set /a I=1
:SINCRO
if not exist %SystemDrive%\Logs\%DATA%_%NomeCliente%_Sincronizzazione_AD_#%I%.log (
	echo Eseguiamo il comando W32tm.exe /monitor ...
	echo +--------------------------------------------------------+  > %SystemDrive%\Logs\%DATA%_%NomeCliente%_Sincronizzazione_AD_#%I%.log
	echo ^|  STATO DI SINCRONIZZAZIONE DEI VARI DOMAIN CONTROLLER  ^| >> %SystemDrive%\Logs\%DATA%_%NomeCliente%_Sincronizzazione_AD_#%I%.log
	echo +--------------------------------------------------------+ >> %SystemDrive%\Logs\%DATA%_%NomeCliente%_Sincronizzazione_AD_#%I%.log
	echo. >> %SystemDrive%\Logs\%DATA%_%NomeCliente%_Sincronizzazione_AD_#%I%.log
	W32tm /monitor >> %SystemDrive%\Logs\%DATA%_%NomeCliente%_Sincronizzazione_AD_#%I%.log
) else (
	set /a I=I+1
	goto SINCRO
)

:STEP8
echo.
echo Controllo di Active Directory terminato.
echo.
echo Per maggiori informazioni controlla i file di log
echo presenti nella cartella %SystemDrive%\Logs
echo.
dir %SystemDrive%\Logs

:END
echo.
endlocal
exit /b